//Copyright 2023 Jerry Heinsen
//Handles all AI Teams.

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class AI_MasterScript : MonoBehaviour
{

//Each AI Team has 3 Members, a Leader and 2 Members.

//Every Team leader always goes straight toward the Center.
//They ignore other Teams.
//The other 2 Members have an intermediate target, followed by the Center.
//These Members can either ignore, avoid or attack other Teams.
//Every AI Team Member has a Strategy in Global.Teams
	public enum enumStrategy { LEADER, IGNORE, AVOID, ATTACK };

//Intermediate Targets.
	private List<Vector3Int> CCW_Targets = new List<Vector3Int>
	{
		new Vector3Int( 4,  7, 0),		//Red
		new Vector3Int(-3,  8, 0),
		new Vector3Int(-8,  1, 0),
		new Vector3Int(-5, -7, 0),
		new Vector3Int( 3, -8, 0),
		new Vector3Int( 7, -1, 0)			//Yellow
	};
	private List<Vector3Int> CW_Targets = new List<Vector3Int>
	{
		new Vector3Int( 7,  1, 0),
		new Vector3Int( 3,  8, 0),
		new Vector3Int(-5,  7, 0),
		new Vector3Int(-8, -1, 0),
		new Vector3Int(-3, -8, 0),
		new Vector3Int( 4, -7, 0)
	};

//OtherMembers 2 tiles away.
	private List<Vector3Int> OM2_OFFSET = new List<Vector3Int>
	{
		new Vector3Int(  0,  2, 0),
		new Vector3Int(  1,  2, 0),
		new Vector3Int(  1,  1, 0),
		new Vector3Int(  2,  0, 0),
		new Vector3Int(  1, -1, 0),
		new Vector3Int(  1, -2, 0),
		new Vector3Int(  0, -2, 0),
		new Vector3Int( -1, -2, 0),
		new Vector3Int( -2, -1, 0),
		new Vector3Int( -2,  0, 0),
		new Vector3Int( -2,  1, 0),
		new Vector3Int( -1,  2, 0),
	};
	private List<Vector3Int> OM2;


//Start here.#############################################
//Set the strategy and target for iTeam.
//This is called from BoardMaster at start.
	public void SetTeam(int iTeam)
	{
	//Leader goes for the center.
		Global.Teams[iTeam,0].strategy = enumStrategy.LEADER;
		Global.Teams[iTeam,0].target = Vector3Int.zero;
		Global.Teams[iTeam,0].hold = false;
	//Others have random Strategy and go for intermediate target.
	//Number 1.
		int iT = iTeam;
		if (iTeam == 0)
			iT = Global.nTEAMS - 1;
		Global.Teams[iTeam,1].strategy = (enumStrategy)Random.Range(1, 4);
		if (Global.Teams[iTeam,1].strategy == enumStrategy.ATTACK)
			Global.Teams[iTeam,1].target = CCW_Targets[iT];
		else
			Global.Teams[iTeam,1].target = CW_Targets[iTeam];
		Global.Teams[iTeam,1].hold = false;
	//Number 2.
		iT = iTeam;
		if (iTeam == Global.nTEAMS-1)
			iT = 0;
		Global.Teams[iTeam,2].strategy = (enumStrategy)Random.Range(1, 4);
		if (Global.Teams[iTeam,2].strategy == enumStrategy.ATTACK)
			Global.Teams[iTeam,2].target = CW_Targets[iT];
		else
			Global.Teams[iTeam,2].target = CCW_Targets[iTeam];
		Global.Teams[iTeam,2].hold = false;
	}

//Move iMember of iTeam to one of the tiles in moveTiles.
//Return the selected moveTile.
//This is called by TurnMaster, and is never the Player.
//There can never be 0 moveTiles.
	public Vector3Int MoveMember(int iTeam, int iMember, List<Vector3Int> moveTiles)
	{
	//Don't move if holding.
		if (Global.Teams[iTeam,iMember].hold)
			return Global.Teams[iTeam,iMember].position;
	//Find the closest moveTile to target.
	//Is Center one of moveTiles?
		Vector3 Target = Global.Grid.CellToWorld(Global.Teams[iTeam,iMember].target);
		float dist = float.MaxValue;
		float D;
		int nMT = 0;
		for (int i = 0; i < moveTiles.Count; i++)
		{
			D = Vector3.Distance(Target, Global.Grid.CellToWorld(moveTiles[i]));
			if (D < dist)
			{
				dist = D;
				nMT = i;
			}
		//If next to Center, don't move.
			if (moveTiles[i] == Vector3Int.zero)
			{
				Global.Teams[iTeam,iMember].hold = true;
				return Global.Teams[iTeam,iMember].position;
			}
		//If next to Target, move there and set Target to Center.
			if (moveTiles[i] == Global.Teams[iTeam,iMember].target)
			{
				Global.Teams[iTeam,iMember].target = Vector3Int.zero;
				return moveTiles[i];
			}
		}
	//Strategy determines move.
	//LEADER and IGNORE take the shortest path.
	//AVOID and ATTACK depend on whether Other Members are near.
		if (Global.Teams[iTeam,iMember].strategy == enumStrategy.LEADER)
			return moveTiles[nMT];
		else if (Global.Teams[iTeam,iMember].strategy == enumStrategy.IGNORE)
			return moveTiles[nMT];
		else
		{
			FindOtherMembers(iTeam, iMember);
			if (OM2.Count > 0)
			{
			//Find the closest Other Member.
				int nOM2 = GetClosest(OM2, Global.Teams[iTeam,iMember].position);
			//Move to the closest moveTile
				if (Global.Teams[iTeam,iMember].strategy == enumStrategy.ATTACK)
					nMT = GetClosest(moveTiles, OM2[nOM2]);
			//Move to the moveTile closest to the target, but not closer to the Other Member.
				else
				{
					if (nMT == nOM2)
					{
						moveTiles.RemoveAt(nMT);
						nMT = GetClosest(moveTiles, Global.Teams[iTeam,iMember].target);
					}
				}
			}
			return moveTiles[nMT];
		}
	}

//Return the index into list of the closest to target.
	private int GetClosest(List<Vector3Int> list, Vector3Int target)
	{
		float dist = float.MaxValue;
		float D = float.MaxValue;
		int n = 0;
		Vector3	Target = Global.Grid.CellToWorld(target);
		for (int i = 0; i < list.Count; i++)
		{
			D = Vector3.Distance(Target, Global.Grid.CellToWorld(list[i]));
			if (D < dist)
			{
				dist = D;
				n = i;
			}
		}
		return n;
	}

//Find other members 2 tiles away for iTeam,iMember
	private void FindOtherMembers(int iTeam, int iMember)
	{
		int ix, iy, ibx, iby;
		OM2 = new List<Vector3Int>();
		Vector3Int center = Global.Teams[iTeam,iMember].position;
		for (int iTile = 0; iTile < 12; iTile++)
		{
			ix = center.x + OM2_OFFSET[iTile].x;
			iy = center.y + OM2_OFFSET[iTile].y;
			ibx = ix + Global.BOARD_OFFSET;
			iby = iy + Global.BOARD_OFFSET;
		//At the start, 2 tiles away will go off the board.
			if ((ibx < 0) || (ibx >= Global.BOARD_SIZE) || (iby < 0) || (iby >= Global.BOARD_SIZE))
				continue;
			if (Global.Board[ibx,iby].tileType == Global.enumTiles.MEMBER)
			{
				if (Global.Board[ibx,iby].iTeam != iTeam)
					OM2.Add(new Vector3Int(ix, iy, 0));
			}
		}
	}

}
