//Copyright 2023 Jerry Heinsen
//Creates the starting Board with Players.

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Tilemaps;

public class BoardMasterScript : MonoBehaviour
{

//The Board is hex shaped and has hex tiles.
//Mark the outside edge Tiles, and 4 triangles of Tiles at the corners, BLOCKED.
//There are 31 internal Tiles also BLOCKED.
//There are 18 Players on Tiles marked MEMBER OR LEADER.
//Everything else is OPEN.

//The Teams.
//Set in Inspector.
//Teams are in order Global.enumTeams
//Members have Leader at index 0, followed by Members.
//On the Board, Leaders have a Black dot, and are between the Members.
	public List<GameObject> Players;

//The Tilemap and Grid.
//Set in Inspector.
	public Tilemap map;
	public GridLayout grid;

//References.
	private AI_MasterScript AIMS;


//Start here.##########################################
//Called from GameMaster.
	public void CreateBoard()
	{
		int ix, iy;
		int t, m;
		int n, iT, k;
		Global.Map = map;
		Global.Grid = grid;
		AIMS = FindObjectOfType<AI_MasterScript>();
	//Mark everything OPEN.
		for (ix = Global.BOARD_MIN_XY; ix < Global.BOARD_MAX_XY+1; ix++)
		{
			for (iy = Global.BOARD_MIN_XY; iy < Global.BOARD_MAX_XY+1; iy++)
			{
				Global.Board[ix+Global.BOARD_OFFSET,iy+Global.BOARD_OFFSET].tileType = Global.enumTiles.OPEN;
			}
		}
	//Center.
		Global.Board[15,15].tileType = Global.enumTiles.CENTER;
	//Interior blocks.
		//Top and bottom.
		Global.Board[13,23].tileType = Global.enumTiles.BLOCKED;
		Global.Board[14,23].tileType = Global.enumTiles.BLOCKED;
		Global.Board[16,23].tileType = Global.enumTiles.BLOCKED;
		Global.Board[17,23].tileType = Global.enumTiles.BLOCKED;
		Global.Board[13, 7].tileType = Global.enumTiles.BLOCKED;
		Global.Board[14, 7].tileType = Global.enumTiles.BLOCKED;
		Global.Board[16, 7].tileType = Global.enumTiles.BLOCKED;
		Global.Board[17, 7].tileType = Global.enumTiles.BLOCKED;
		//Left.
		Global.Board[ 8,17].tileType = Global.enumTiles.BLOCKED;
		Global.Board[ 8,18].tileType = Global.enumTiles.BLOCKED;
		Global.Board[ 9,20].tileType = Global.enumTiles.BLOCKED;
		Global.Board[10,21].tileType = Global.enumTiles.BLOCKED;
		Global.Board[ 8,13].tileType = Global.enumTiles.BLOCKED;
		Global.Board[ 8,12].tileType = Global.enumTiles.BLOCKED;
		Global.Board[ 9,10].tileType = Global.enumTiles.BLOCKED;
		Global.Board[10, 9].tileType = Global.enumTiles.BLOCKED;
		//Right.
		Global.Board[22,17].tileType = Global.enumTiles.BLOCKED;
		Global.Board[21,18].tileType = Global.enumTiles.BLOCKED;
		Global.Board[20,20].tileType = Global.enumTiles.BLOCKED;
		Global.Board[20,21].tileType = Global.enumTiles.BLOCKED;
		Global.Board[22,13].tileType = Global.enumTiles.BLOCKED;
		Global.Board[21,12].tileType = Global.enumTiles.BLOCKED;
		Global.Board[20,10].tileType = Global.enumTiles.BLOCKED;
		Global.Board[20, 9].tileType = Global.enumTiles.BLOCKED;
		//6 Tiles closer to the center.
		Global.Board[15,19].tileType = Global.enumTiles.BLOCKED;
		Global.Board[18,17].tileType = Global.enumTiles.BLOCKED;
		Global.Board[18,13].tileType = Global.enumTiles.BLOCKED;
		Global.Board[15,11].tileType = Global.enumTiles.BLOCKED;
		Global.Board[12,13].tileType = Global.enumTiles.BLOCKED;
		Global.Board[12,17].tileType = Global.enumTiles.BLOCKED;
	//Corner triangles.
		for (ix = Global.BOARD_MIN_XY; ix < Global.BOARD_MAX_XY+1; ix++)
		{
			for (iy = 0; iy < Global.BOARD_MAX_XY+2; iy++)
			{
				if ((iy == 1) && (ix > 13))
					BlockTiles(ix, iy);
				else if ((iy == 2) && ((ix > 13) || (ix < -13)))
					BlockTiles(ix, iy);
				else if ((iy == 3) && ((ix > 12) || (ix < -13)))
					BlockTiles(ix, iy);
				else if ((iy == 4) && ((ix > 12) || (ix < -12)))
					BlockTiles(ix, iy);
				else if ((iy == 5) && ((ix > 11) || (ix < -12)))
					BlockTiles(ix, iy);
				else if ((iy == 6) && ((ix > 11) || (ix < -11)))
					BlockTiles(ix, iy);
				else if ((iy == 7) && ((ix > 10) || (ix < -11)))
					BlockTiles(ix, iy);
				else if ((iy == 8) && ((ix > 10) || (ix < -10)))
					BlockTiles(ix, iy);
				else if ((iy == 9) && ((ix > 9) || (ix < -10)))
					BlockTiles(ix, iy);
				else if ((iy == 10) && ((ix > 9) || (ix < -9)))
					BlockTiles(ix, iy);
				else if ((iy == 11) && ((ix > 8) || (ix < -9)))
					BlockTiles(ix, iy);
				else if ((iy == 12) && ((ix > 8) || (ix < -8)))
					BlockTiles(ix, iy);
				else if ((iy == 13) && ((ix > 7) || (ix < -8)))
					BlockTiles(ix, iy);
				else if ((iy == 14) && ((ix > 7) || (ix < -7)))
					BlockTiles(ix, iy);
				else if (iy == 15)
					BlockTiles(ix, iy);
			}
		}
	//Player positions.
	//Upper Right.
		Global.Teams[0,0].position = new Vector3Int(10, 7, 0);
		Global.Teams[0,1].position = new Vector3Int(11, 6, 0);
		Global.Teams[0,2].position = new Vector3Int(10, 8, 0);
	//Top.
		Global.Teams[1,0].position = new Vector3Int( 0, 14, 0);
		Global.Teams[1,1].position = new Vector3Int( 1, 14, 0);
		Global.Teams[1,2].position = new Vector3Int(-1, 14, 0);
	//Upper Left.
		Global.Teams[2,0].position = new Vector3Int(-11, 7, 0);
		Global.Teams[2,1].position = new Vector3Int(-10, 8, 0);
		Global.Teams[2,2].position = new Vector3Int(-11, 6, 0);
	//Lower Left.
		Global.Teams[3,0].position = new Vector3Int(-11, -7, 0);
		Global.Teams[3,1].position = new Vector3Int(-11, -6, 0);
		Global.Teams[3,2].position = new Vector3Int(-10, -8, 0);
	//Bottom.
		Global.Teams[4,0].position = new Vector3Int( 0, -14, 0);
		Global.Teams[4,1].position = new Vector3Int(-1, -14, 0);
		Global.Teams[4,2].position = new Vector3Int( 1, -14, 0);
	//Lower Right.
		Global.Teams[5,0].position = new Vector3Int(10, -7, 0);
		Global.Teams[5,1].position = new Vector3Int(10, -8, 0);
		Global.Teams[5,2].position = new Vector3Int(11, -6, 0);
	//Set Tile type and Teams properties.
		for (t = 0; t < Global.nTEAMS; t++)
		{
			for (m = 0; m < Global.nMEMBERS; m++)
			{
				ix = Global.Teams[t,m].position.x + Global.BOARD_OFFSET;
				iy = Global.Teams[t,m].position.y + Global.BOARD_OFFSET;
				Global.Board[ix,iy].tileType = Global.enumTiles.MEMBER; 
				Global.Board[ix,iy].iTeam = t; 
				Global.Board[ix,iy].iMember = m; 
				n = (3 * t) + m;
				Global.Teams[t,m].alive = true;
				Global.Teams[t,m].GO = Players[n];
				Global.Teams[t,m].SR = Global.Teams[t,m].GO.GetComponent<SpriteRenderer>();
			}
		//AI Team Members have a strategy.
			if (t != Global.iPlayer)
				AIMS.SetTeam(t);
		}
	//Set Teams.
		List<int> iTeam = new List<int> { 0, 1, 2, 3, 4, 5 };
		for (t = 0; t < Global.nTEAMS; t++)
		{
			iT = Random.Range(0, iTeam.Count);
			for (m = 0; m < Global.nMEMBERS; m++)
			{
				k = iTeam[iT];
				Global.Teams[k,m].GO.transform.position = Global.Grid.CellToWorld(Global.Teams[k,m].position);
			}
			iTeam.RemoveAt(iT);
		}
	}

//Block tiles.
	private void BlockTiles(int ix, int iy)
	{
		Global.Board[ix+Global.BOARD_OFFSET,iy+Global.BOARD_OFFSET].tileType = Global.enumTiles.BLOCKED;
		Global.Board[ix+Global.BOARD_OFFSET,-iy+Global.BOARD_OFFSET].tileType = Global.enumTiles.BLOCKED;
	}

}
