//Copyright 2023 Jerry Heinsen
//Starts and stops the game.

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class GameMasterScript : MonoBehaviour
{

//Canvases.
	public GameObject startCanvas;
	public GameObject endCanvas;
	public Text endDescriptionText;

//Player Color.
	public Toggle RedToggle;
	public Toggle MagentaToggle;
	public Toggle BlueToggle;
	public Toggle CyanToggle;
	public Toggle GreenToggle;
	public Toggle YellowToggle;

//Player MaxTime.
	public Toggle FiveSecondToggle;
	public Toggle TenSecondToggle;

//AI AvgTime.
	public Toggle ShortAvgToggle;
	public Toggle LongAvgToggle;

//References.
	private BoardMasterScript BMS;
	private TurnMasterScript TMS;
	private AudioSource AS;
	private ShowTilePositionScript STPS;


//Start here.################################################
//The Start Canvas is displayed.
//Then wait for the Start Button pressed.
	private void Start()
	{
		BMS = FindObjectOfType<BoardMasterScript>();
		TMS = FindObjectOfType<TurnMasterScript>();
		AS = GetComponent<AudioSource>();
		STPS = GetComponent<ShowTilePositionScript>();
		startCanvas.SetActive(true);
		LoadPersistantItems();
	}

//Check for escape.
	private void Update()
	{
		if (Global.DoNotRun)
			return;
		if (Input.GetKey(KeyCode.Escape))
			EndGame(Global.enumEndGame.ESCAPE);
	}

//Start game.
	public void StartGame()
	{
		startCanvas.SetActive(false);
		SetPersistantItems();
		BMS.CreateBoard();
		//STPS.DoNotShow = false;				//To show Tile positions with left click, un-comment this,
		TMS.BeginGame();								//and comment this, Tile X,Y are shown in GameMaster Inspector.
		//AS.Play();
		Global.DoNotRun = false;
	}

//End game.
//The game can end in various ways.
	public void EndGame(Global.enumEndGame endGame)
	{
		Global.DoNotRun = true;
		endDescriptionText.text = Global.EndGameStrings[(int)endGame];
		Invoke("DelayEnd", 2f);
	}

//Delay before showing the endCanvas.
	private void DelayEnd()
	{
		endCanvas.SetActive(true);
		SavePersistantItems();
		Invoke("AllDone", 4f);
	}

//Really End Game after delay.
	public void AllDone()
	{
		#if UNITY_EDITOR
			UnityEditor.EditorApplication.isPlaying = false;
		#endif
		Application.Quit();
	}


//Persistant Items.##########################################
//Load at start.
	private void LoadPersistantItems()
	{
	//Max Time.
		Global.PlayerMaxTime = PlayerPrefs.GetFloat("PlayerMaxTime", 5f);
		if (Global.PlayerMaxTime == 5f)
			FiveSecondToggle.isOn = true;
		else if (Global.PlayerMaxTime == 10f)
			TenSecondToggle.isOn = true;
	//Avg Time.
		Global.AI_AvgTime = PlayerPrefs.GetFloat("AI_AvgTime", 0.4f);
		if (Global.AI_AvgTime == 0.4f)
			ShortAvgToggle.isOn = true;
		else if (Global.AI_AvgTime == 0.8f)
			LongAvgToggle.isOn = true;
	//Color.
		Global.iPlayer = PlayerPrefs.GetInt("iPlayer", 0);
		if (Global.iPlayer == (int)Global.enumTeams.RED)
			RedToggle.isOn = true;
		else if (Global.iPlayer == (int)Global.enumTeams.MAGENTA)
			MagentaToggle.isOn = true;
		else if (Global.iPlayer == (int)Global.enumTeams.BLUE)
			BlueToggle.isOn = true;
		else if (Global.iPlayer == (int)Global.enumTeams.CYAN)
			CyanToggle.isOn = true;
		else if (Global.iPlayer == (int)Global.enumTeams.GREEN)
			GreenToggle.isOn = true;
		else if (Global.iPlayer == (int)Global.enumTeams.YELLOW)
			YellowToggle.isOn = true;
	}

//Set from Toggles.
	private void SetPersistantItems()
	{
	//Max Time.
		if (FiveSecondToggle.isOn)
			Global.PlayerMaxTime = 5f;
		else if (TenSecondToggle.isOn)
			Global.PlayerMaxTime = 10f;
	//Max Time.
		if (ShortAvgToggle.isOn)
			Global.AI_AvgTime = 0.4f;
		else if (LongAvgToggle.isOn)
			Global.AI_AvgTime = 0.8f;
	//Color.
		if (RedToggle.isOn)
			Global.iPlayer = (int)Global.enumTeams.RED;
		else if (MagentaToggle.isOn)
			Global.iPlayer = (int)Global.enumTeams.MAGENTA;
		else if (BlueToggle.isOn)
			Global.iPlayer = (int)Global.enumTeams.BLUE;
		else if (CyanToggle.isOn)
			Global.iPlayer = (int)Global.enumTeams.CYAN;
		else if (GreenToggle.isOn)
			Global.iPlayer = (int)Global.enumTeams.GREEN;
		else if (YellowToggle.isOn)
			Global.iPlayer = (int)Global.enumTeams.YELLOW;
	}

//Save at end.
	private void SavePersistantItems()
	{
		PlayerPrefs.SetFloat("PlayerMaxTime", Global.PlayerMaxTime);
		PlayerPrefs.SetFloat("AI_AvgTime", Global.AI_AvgTime);
		PlayerPrefs.SetInt("iPlayer", Global.iPlayer);
	}

}
