//Copyright 2023 Jerry Heinsen
//Everything here is available to all Scripts.
//This Project was written with:  Indenting - none, Tab size = 2, Keep Tabs.
//Set in the VS menu: Tools - Options - Text Editor - C# - Tabs

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Tilemaps;

public class Global : MonoBehaviour
{

//Game.#################################################
//This is a 2D top-down turn-based Game.
//It is played on a hex Tilemap board with a hex grid.
	public static Tilemap Map;
	public static GridLayout Grid;
//There are 6 Teams, each with 3 Members, one the Leader.
//The Player runs one Team, all others are AI.

//Win by getting any 2 Team Members next to the Center.

//All Team Members can move one Tile in any of 6 directions.
//There is a wall around, 30 interior obstacles, and 1 center Tile, all are BLOCKED.
//Any Member moving next to a opponent Member causes:
//	A Leader against a Leader, or a Member against a Member is 60-40 for the mover.
//	A leader against a Member is 70-30 for the Leader.
//	Any Member against 2 or 3 opponent Members is dead.


//Start here.###########################################
//Do not let any Updates or FixedUpdates run until start button is pressed.
	public static bool DoNotRun = true;

//The game may end in a number of ways.
	public enum enumEndGame { ESCAPE, LOSE, AI_WIN, WIN };
	public enumEndGame EndGame;
	public static List<string> EndGameStrings = new List<string>
	{
		{
			"The game has been ended with the ESC key.\n" +
			"This has to be considered incomplete..."
		},
		{
			"Your Team has lost 2 Players, so you can not win.\n" +
			"Possibly you were too agressive..."
		},
		{
			"An AI Team has Taken the Center.\n" +
			"They think they are better than you..."
		},
		{
			"You have Taken the Center !\n" +
			"You are awarded 1 minute of feeling Greatly Superior to AI."
		}
	};


//Teams.################################################
//There are six Teams, each with a different color.
	public const int nTEAMS = 6;
	public enum enumTeams { RED, MAGENTA, BLUE, CYAN, GREEN, YELLOW };
//Each Team has 3 members.
	public const int nMEMBERS = 3;
//Teams[n,0] is always the Leader.

//A single Team member.
	public struct strucTeamMember
	{
		public bool alive;
		public GameObject GO;
		public Vector3Int position;		//Tile coordinates
		public SpriteRenderer SR;
	//Only used by AI Team Members.
		public AI_MasterScript.enumStrategy strategy;
		public Vector3Int target;
		public bool hold;							//hold this position
	};
//All Teams.
	public static strucTeamMember[,] Teams = new strucTeamMember[nTEAMS, nMEMBERS];

//One Team is the Player.
//The Player is run using the Mouse Left-click.
	public static int iPlayer;
	public static float PlayerMaxTime;

//All others are AI.
//All AI Teams are run by AI_Master.
	public static float AI_AvgTime;


//Board.################################################
//The Board is a hex shaped TileMap with a hex grid.
//The origin is at the Board center, Tiles are point up.
//Tile indices are Vector3int, and are not related to the size of the Tile.
	public const int BOARD_SIZE = 31;
	public enum enumTiles { CENTER, OPEN, BLOCKED, MEMBER };
	public struct strucTile
	{
		public enumTiles tileType;
		public int iTeam;						//these 2 for MEMBER or PLAYER only
		public int iMember;					//0 is the Leader
	};
//Board.
	public static strucTile[,] Board = new strucTile[BOARD_SIZE, BOARD_SIZE];
//The grid coordinates are -15 to +15 on each axis.
//The array coordinates are 0 to 31.
//When addressing the array add this offset to each index.
	public const int BOARD_OFFSET = 15;
	public static Vector3Int BoardOffset = new Vector3Int(BOARD_OFFSET, BOARD_OFFSET, 0);

//The Player left-clicks to select a Tile.
//Everything outside the Board is not legal.
	public const int BOARD_MIN_XY = -14;
	public const int BOARD_MAX_XY = 14;

//When it is a Members Turn, there are 6 Tiles surrounding them,
//that are possible to move to.
//Odd and even refer to the center tile y.
	public static List<Vector3Int> EVEN_MOVE_OFFSET = new List<Vector3Int>
	{
		new Vector3Int( 1,  0, 0),		//right
		new Vector3Int( 0,  1, 0),		//right above
		new Vector3Int(-1,  1, 0),		//left above
		new Vector3Int(-1,  0, 0),		//left
		new Vector3Int(-1, -1, 0),		//left below
		new Vector3Int( 0, -1, 0)			//right below
	};
	public static List<Vector3Int> ODD_MOVE_OFFSET = new List<Vector3Int>
	{
		new Vector3Int( 1,  0, 0),		//right
		new Vector3Int( 1,  1, 0),		//right above
		new Vector3Int( 0,  1, 0),		//left above
		new Vector3Int(-1,  0, 0),		//left
		new Vector3Int( 0, -1, 0),		//left below
		new Vector3Int( 1, -1, 0)			//right below
	};
//This is the Tiles around the Center (0,0,0)
	public static List<Vector3Int> CENTER_MOVE_OFFSET = EVEN_MOVE_OFFSET;

}
