//Copyright 2023 Jerry Heinsen
//Handles all Team movement.

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Tilemaps;

public class TurnMasterScript : MonoBehaviour
{

//There are 5 Teams run by AI, and one run by the Player.

//Each of the AI Teams has a delay in movement, to not appear instantaneous.
//The timout value is random around Global.AI_AvgTime
//Members are moved in their Global.Teams order, 0 to 2.

//The Player uses the Mouse to move.
//Player also has Global.PlayerMaxTime timeout, if Mouse doesn't happen.
//Like AI, Members are moved in their Global.Teams order.

//Each Member is darkened in their turn.
	private Color DarkColor = new Color(0.3f, 0.3f, 0.3f, 1f);

//The Tiles that are legal to move to, are highlighted for the Player only.
	public Tile HighlightTile;
	public Tile NormalTile;

//Current Turn.
	private int iTeam;
	private int iMember;
	private float turnTime;
	private float endTime;
	private Color oldColor;
	private List<Vector3Int> moveTiles;
//The Turns are run by a simple FSM.
	private enum enumState { START, AI, PLAYER, END };
	private enumState State;

//Explosion when Member is destroyed.
//Ding when it's Player move.
	public AudioSource AS;
	public ParticleSystem Explosion;
	public AudioClip Bang;
	public AudioClip Ding;

//Beat time increases as Player closer to center.
	public AudioSource Bump_AS;
	public AudioClip AmbBump;
	private float BumpTime;
	private float BumpInterval;
	private const float MAX_BUMP_INTERVAL = 0.85f;
	private const float DELTA_INTERVAL = -0.015f;
	private int BumpTeam;

//References.
	private GameMasterScript GMS;
	private AI_MasterScript AIMS;


//Start here.##########################################
//This is called from GameMaster.
	public void BeginGame()
	{
		GMS = FindObjectOfType<GameMasterScript>();
		AIMS = FindObjectOfType<AI_MasterScript>();
	//Start with random Team.
		endTime = 0f;
		iTeam = Random.Range(0, Global.nTEAMS);
		State = enumState.START;
	//Start Bump.
		BumpInterval = MAX_BUMP_INTERVAL;
		BumpTime = Time.time + BumpInterval;
		BumpTeam = 0;
		if (Global.iPlayer == 0)
			BumpTeam = 1;
	}

//Physics Update runs the Game.
	private void FixedUpdate()
	{
		if (Global.DoNotRun)
			return;
	//As long as the Player has at least 2 Memebers, they can win.
		int n = 0;
		for (int m = 0; m < Global.nMEMBERS; m++)
		{
			if (Global.Teams[Global.iPlayer,m].alive)
				n += 1;
		}
	//Game is over.
		if (n < 2)
			GMS.EndGame(Global.enumEndGame.LOSE);
	//OK to continue.
		else
		{
			DoState();
			if (Time.time > BumpTime)
			{
				Bump_AS.volume = Random.Range(0.03f, 0.1f);
				Bump_AS.PlayOneShot(AmbBump);
				BumpTime = Time.time + BumpInterval;
				if (iTeam == BumpTeam)
					BumpInterval += DELTA_INTERVAL;
			}
		}
	}

//Do something useful.
	private void DoState()
	{
		switch(State)
		{
		//Start a new Team.
			case enumState.START:
			//Are any Members alive ?
				for (int m = 0; m < Global.nMEMBERS; m++)
				{
				//Yes.
					if (Global.Teams[iTeam,m].alive)
					{
						if (iTeam == Global.iPlayer)
						{
							State = enumState.PLAYER;
							turnTime = Global.PlayerMaxTime;
							AS.volume = 0.7f;
							AS.PlayOneShot(Ding);
						}
						else
						{
							State = enumState.AI;
							turnTime = Random.Range(Global.AI_AvgTime-0.2f, Global.AI_AvgTime+0.3f);
						}
						endTime = Time.time + turnTime;
						iMember = m;
						oldColor = Global.Teams[iTeam,iMember].SR.color;
						Global.Teams[iTeam,iMember].SR.color = DarkColor;
						FindMoveTiles();
						return;
					}
				}
			//No.
				State = enumState.END;
				break;
		//Run the AI Team.
			case enumState.AI:
				if (Time.time > endTime)
				{
					MoveMember(AIMS.MoveMember(iTeam, iMember, moveTiles));
					NextMember();
				}
				break;
		//Run the Player Team.
			case enumState.PLAYER:
				if (Time.time > endTime)
					NextMember();
				break;
		//Next Team.
			case enumState.END:
				if (iTeam == Global.iPlayer)
					SetMoveTiles(NormalTile);
				iTeam += 1;
				if (iTeam >= Global.nTEAMS)
					iTeam = 0;
				State = enumState.START;
				break;
		}
	}

//Start the Next Member.
	private void NextMember()
	{
	//Restore where we were.
		Global.Teams[iTeam,iMember].SR.color = oldColor;
		SetMoveTiles(NormalTile);
	//Next Member.
		for (int m = iMember+1; m < Global.nMEMBERS; m++)
		{
			if (Global.Teams[iTeam,m].alive)
			{
				endTime = Time.time + turnTime;
				iMember = m;
				Global.Teams[iTeam,iMember].SR.color = DarkColor;
				FindMoveTiles();
				if (iTeam == Global.iPlayer)
				{
					AS.volume = 0.7f;
					AS.PlayOneShot(Ding);
				}
				return;
			}
		}
	//There are no more Members.
		State = enumState.END;
	}

//Update is only used for the Player.
	private void Update()
	{
		if (Global.DoNotRun)
			return;
		if (iTeam != Global.iPlayer)
			return;
	//left click.
		if (Input.GetMouseButtonDown(0))
		{
			Vector3 p = Camera.main.ScreenToWorldPoint(Input.mousePosition);
			p.z = 0f;
			Vector3Int ip = Global.Grid.WorldToCell(p);
		//The click has to be in moveTiles.
			for (int i = 0; i < moveTiles.Count; i++)
			{
				if (ip == moveTiles[i])
				{
					MoveMember(ip);
					NextMember();
					break;
				}
			}
		}
	}

//Find the moveTiles for iTeam,iMember
	private void FindMoveTiles()
	{
		int ix, iy;
		moveTiles = new List<Vector3Int>();
		Vector3Int center = Global.Teams[iTeam,iMember].position;
		List<Vector3Int> moveOffset = new List<Vector3Int>();
		if ((center.y % 2) == 0)
			moveOffset = Global.EVEN_MOVE_OFFSET;
		else
			moveOffset = Global.ODD_MOVE_OFFSET;
		for (int iTile = 0; iTile < 6; iTile++)
		{
			ix = center.x + moveOffset[iTile].x;
			iy = center.y + moveOffset[iTile].y;
			if (Global.Board[ix+Global.BOARD_OFFSET,iy+Global.BOARD_OFFSET].tileType == Global.enumTiles.OPEN)
			{
				moveTiles.Add(new Vector3Int(ix, iy, 0));
				if (iTeam == Global.iPlayer)
					SetMoveTiles(HighlightTile);
			}
		}
	}

//Set moveTiles to tile.
//This is only used by the Player.
	private void SetMoveTiles(Tile tile)
	{
		for (int iTile = 0; iTile < moveTiles.Count; iTile++)
		{
			Global.Map.SetTile(moveTiles[iTile], tile);
		}
	}

//Move iTeam,iMember to ip.
	private void MoveMember(Vector3Int ip)
	{
	//Remove current position.
		Vector3Int center = Global.Teams[iTeam,iMember].position + Global.BoardOffset;
		Global.Board[center.x,center.y].tileType = Global.enumTiles.OPEN;
	//Move to new.
		Vector3Int move = ip + Global.BoardOffset;
		Global.Board[move.x,move.y].tileType = Global.enumTiles.MEMBER;
		Global.Board[move.x,move.y].iTeam = iTeam;
		Global.Board[move.x,move.y].iMember = iMember;
		Global.Teams[iTeam,iMember].position = ip;
		Global.Teams[iTeam,iMember].GO.transform.position = Global.Grid.CellToWorld(ip);
		Global.Teams[iTeam,iMember].SR.color = oldColor;
		endTime = 0f;
	//Check if Member has moved next to another Teams Member.
		CheckMemberConflict();
	//Check if iTeam is a winner.
		CheckTeamWin();
	}

//Check if iTeam has 2 Members next to Center.
//If they do, it is a win for the Team, and ends Game.
	private void CheckTeamWin()
	{
		int ix, iy;
		int nMembers = 0;
		for (int iTile = 0; iTile < 6; iTile++)
		{
			ix = Global.CENTER_MOVE_OFFSET[iTile].x + Global.BOARD_OFFSET;
			iy = Global.CENTER_MOVE_OFFSET[iTile].y + Global.BOARD_OFFSET;
			if (Global.Board[ix,iy].tileType == Global.enumTiles.MEMBER)
			{
				if (Global.Board[ix,iy].iTeam == iTeam)
					nMembers += 1;
			}
		}
		if (nMembers > 1)
		{
			if (iTeam == Global.iPlayer)
				GMS.EndGame(Global.enumEndGame.WIN);
			else
				GMS.EndGame(Global.enumEndGame.AI_WIN);
		}
	}


//Member destruction.############################################
//Check if iTeam,iMember has moved next to Other Teams Member(s).
	private void CheckMemberConflict()
	{
		Vector3Int center = Global.Teams[iTeam,iMember].position;
		List<Vector3Int> moveOffset = new List<Vector3Int>();
		if ((center.y % 2) == 0)
			moveOffset = Global.EVEN_MOVE_OFFSET;
		else
			moveOffset = Global.ODD_MOVE_OFFSET;
	//There are 6 possible locations around iMember.
	//Get any Members of Other Teams.
		List<Global.strucTile> locations = new List<Global.strucTile>();
		int ix, iy;
		for (int iTile = 0; iTile < 6; iTile++)
		{
			ix = center.x + moveOffset[iTile].x + Global.BOARD_OFFSET;
			iy = center.y + moveOffset[iTile].y + Global.BOARD_OFFSET;
			if (Global.Board[ix,iy].tileType == Global.enumTiles.MEMBER)
			{
				if (Global.Board[ix,iy].iTeam != iTeam)
					locations.Add(Global.Board[ix,iy]);
			}
		}
	//No Conflict.
		if (locations.Count == 0)
			return;
	//One Other means a someone will die.
		else if (locations.Count == 1)
			DestroyMember(locations[0].iTeam, locations[0].iMember);
	//Two Others can be the same Team, in which case the mover dies.
	//Or they can be on different Teams, which may require 2 Conflicts.
		else if (locations.Count == 2)
		{
			if (locations[0].iTeam == locations[1].iTeam)
				KillMember(locations[0].iTeam, locations[0].iMember, 0);		//death on iTeam
			else
			{
				int n = Random.Range(0, 2);
				DestroyMember(locations[n].iTeam, locations[n].iMember);
				if (Global.Teams[iTeam,iMember].alive)
				{
					if (n == 0)
						n = 1;
					else
						n = 0;
					DestroyMember(locations[n].iTeam, locations[n].iMember);
				}
			}
		}
	//Three Others can be 3 on the same Team, or 2 and 1.
	//In either case the mover is dead.
		else if (locations.Count == 3)
			KillMember(locations[0].iTeam, locations[0].iMember, 0);		//death on iTeam
	}

//Destroy a Member when iTeam,iMember is the mover.
	private void DestroyMember(int otherTeam, int otherMember)
	{
		if (otherMember == 0)
		{
			if (iMember == 0)
				KillMember(otherTeam, otherMember, 6);		// 60/40 in my favor
			else
				KillMember(otherTeam, otherMember, 3);		// 70/30 in their favor
		}
		else
		{
			if (iMember == 0)
				KillMember(otherTeam, otherMember, 7);		// 70/30 in my favor
			else
			{
				KillMember(otherTeam, otherMember, 6);		// 60/40 in my favor
			}
		}
	}

//Kill a Team Member.
	private void KillMember(int otherTeam, int otherMember, int n)
	{
		int ix, iy;
		Vector3 explosion;
		if (Random.Range(0, 10) < n)
		{
		//Kill the Other.
			explosion = Global.Grid.CellToWorld(Global.Teams[otherTeam,otherMember].position);
			ix = Global.Teams[otherTeam,otherMember].position.x + Global.BOARD_OFFSET;
			iy = Global.Teams[otherTeam,otherMember].position.y + Global.BOARD_OFFSET;
			Global.Board[ix,iy].tileType = Global.enumTiles.OPEN;
			Global.Teams[otherTeam,otherMember].alive = false;
			Global.Teams[otherTeam,otherMember].GO.SetActive(false);
		}
		else
		{
		//Commit Suicide.
			explosion = Global.Grid.CellToWorld(Global.Teams[iTeam,iMember].position);
			ix = Global.Teams[iTeam,iMember].position.x + Global.BOARD_OFFSET;
			iy = Global.Teams[iTeam,iMember].position.y + Global.BOARD_OFFSET;
			Global.Board[ix,iy].tileType = Global.enumTiles.OPEN;
			Global.Teams[iTeam,iMember].alive = false;
			Global.Teams[iTeam,iMember].GO.SetActive(false);
		}
		Explosion.gameObject.transform.position = explosion;
		Explosion.Play();
		AS.volume = 0.7f;
		AS.PlayOneShot(Bang);
	}

}
